@@ -4,8 +4,7 @@ import os
 from typing import Literal
 
 import numpy as np
-from numpy import (outer, cos, sin, ones)
-
+from numpy import cos, ones, outer, sin
 from packaging.version import parse as parse_version
 
 from . import Qobj, expect, sigmax, sigmay, sigmaz
@@ -13,9 +12,8 @@ from . import Qobj, expect, sigmax, sigmay, sigmaz
 try:
     import matplotlib
     import matplotlib.pyplot as plt
-    from mpl_toolkits.mplot3d import Axes3D
     from matplotlib.patches import FancyArrowPatch
-    from mpl_toolkits.mplot3d import proj3d
+    from mpl_toolkits.mplot3d import Axes3D, proj3d
 
     # Define a custom _axes3D function based on the matplotlib version.
     # The auto_add_to_figure keyword is new for matplotlib>=3.4.
@@ -55,6 +53,12 @@ except ImportError:
     pass
 
 
+def _state_to_cartesian_coordinates(state: Qobj):
+    return [expect(sigmax(), state),
+            expect(sigmay(), state),
+            expect(sigmaz(), state)]
+
+
 class Bloch:
     r"""
     Class for plotting data on the Bloch sphere.  Valid data can be either
@@ -366,7 +370,8 @@ class Bloch:
         self.point_alpha.append(alpha)
         self._inner_point_color.append(colors)
 
-    def add_states(self, state, kind: Literal['vector', 'point'] = 'vector',
+    def add_states(self, state: Qobj,
+                   kind: Literal['vector', 'point'] = 'vector',
                    colors=None, alpha=1.0):
         """Add a state vector Qobj to Bloch sphere.
 
@@ -378,8 +383,9 @@ class Bloch:
         kind : {'vector', 'point'}
             Type of object to plot.
 
-        colors : array_like
+        colors : str or array_like
             Optional array with colors for the states.
+            The colors can be a string or a RGB or RGBA tuple.
 
         alpha : float, default=1.
             Transparency value for the vectors. Values between 0 and 1.
@@ -397,24 +403,36 @@ class Bloch:
             colors = np.asarray(colors)
 
             if colors.ndim == 0:
-                colors = colors[np.newaxis]
+                colors = np.repeat(colors, state.shape[0])
 
-            if colors.shape != state.shape:
+            elif (
+                colors.ndim == 1
+                and (np.issubdtype(colors.dtype, np.floating)
+                     or np.issubdtype(colors.dtype, np.integer))):
+                colors = colors[np.newaxis]
+                colors = np.repeat(colors, [state.shape[0]], axis=0)
+
+            if (
+                colors.shape[0] != state.shape[0] or colors.ndim > 2
+                or colors.ndim == 2
+                and not (np.issubdtype(colors.dtype, np.floating)
+                         or np.issubdtype(colors.dtype, np.integer))):
                 raise ValueError("The included colors are not valid. "
-                                 "colors must be equivalent to a 1D array "
-                                 "with the same size as the number of states.")
+                                 "colors must have the same size as state.")
+
         else:
-            colors = np.array([None] * state.size)
+            colors = np.array([None] * state.shape[0])
 
         for k, st in enumerate(state):
-            vec = [expect(sigmax(), st),
-                   expect(sigmay(), st),
-                   expect(sigmaz(), st)]
+            vec = _state_to_cartesian_coordinates(st)
 
             if kind == 'vector':
                 self.add_vectors(vec, colors=[colors[k]], alpha=alpha)
             elif kind == 'point':
                 self.add_points(vec, colors=[colors[k]], alpha=alpha)
+            else:
+                raise ValueError("The included kind is not valid. "
+                                 f"It should be vector or point, not {kind}.")
 
     def add_vectors(self, vectors, colors=None, alpha=1.0):
         """Add a list of vectors to Bloch sphere.
@@ -424,8 +442,9 @@ class Bloch:
         vectors : array_like
             Array with vectors of unit length or smaller.
 
-        colors : array_like
+        colors : str or array_like
             Optional array with colors for the vectors.
+            The colors can be a string or a RGB or RGBA tuple.
 
         alpha : float, default=1.
             Transparency value for the vectors. Values between 0 and 1.
@@ -443,16 +462,22 @@ class Bloch:
                 "index represents the iteration over the vectors and the "
                 "second index represents the position in 3D of vector head.")
 
-        n_vectors = vectors.shape[0]
         if colors is None:
-            colors = np.array([None] * n_vectors)
+            colors = np.array([None] * vectors.shape[0])
         else:
             colors = np.asarray(colors)
 
-        if colors.ndim != 1 or colors.size != n_vectors:
-            raise ValueError("The included colors are not valid. colors must "
-                             "be equivalent to a 1D array with the same "
-                             "size as the number of vectors. ")
+            if colors.ndim == 0:
+                colors = np.repeat(colors, vectors.shape[0])
+
+            if (
+                colors.shape[0] != vectors.shape[0]
+                or colors.ndim == 2
+                and not (np.issubdtype(colors.dtype, np.floating)
+                         or np.issubdtype(colors.dtype, np.integer))
+            ):
+                raise ValueError("The included colors are not valid. "
+                                 "colors must have the same size as vectors.")
 
         for k, vec in enumerate(vectors):
             self.vectors.append(vec)
@@ -483,9 +508,7 @@ class Bloch:
 
         """
         if isinstance(state_or_vector, Qobj):
-            vec = [expect(sigmax(), state_or_vector),
-                   expect(sigmay(), state_or_vector),
-                   expect(sigmaz(), state_or_vector)]
+            vec = _state_to_cartesian_coordinates(state_or_vector)
         elif isinstance(state_or_vector, (list, np.ndarray, tuple)) \
                 and len(state_or_vector) == 3:
             vec = state_or_vector
@@ -523,42 +546,36 @@ class Bloch:
             Additional parameters to pass to the matplotlib .plot function
             when rendering this arc.
         """
-        if isinstance(start, Qobj):
-            pt1 = [
-                expect(sigmax(), start),
-                expect(sigmay(), start),
-                expect(sigmaz(), start),
-            ]
-        else:
-            pt1 = start
+        pt1 = _state_to_cartesian_coordinates(start) \
+            if isinstance(start, Qobj) else start
 
-        if isinstance(end, Qobj):
-            pt2 = [
-                expect(sigmax(), end),
-                expect(sigmay(), end),
-                expect(sigmaz(), end),
-            ]
-        else:
-            pt2 = end
+        pt2 = _state_to_cartesian_coordinates(end) \
+            if isinstance(end, Qobj) else end
 
         pt1 = np.asarray(pt1)
         pt2 = np.asarray(pt2)
 
         len1 = np.linalg.norm(pt1)
         len2 = np.linalg.norm(pt2)
-        if len1 < 1e-12 or len2 < 1e-12:
-            raise ValueError('Polar and azimuthal angles undefined at origin.')
-        elif abs(len1 - len2) > 1e-12:
-            raise ValueError("Points not on the same sphere.")
-        elif (pt1 == pt2).all():
-            raise ValueError(
-                "Start and end represent the same point. No arc can be formed."
-            )
-        elif (pt1 == -pt2).all():
-            raise ValueError(
-                "Start and end are diagonally opposite, no unique arc is"
-                " possible."
-            )
+
+        def check_arguments_validity():
+            if len1 < 1e-12 or len2 < 1e-12:
+                raise ValueError("Polar and azimuthal angles undefined"
+                                 " at origin.")
+            elif abs(len1 - len2) > 1e-12:
+                raise ValueError("Points not on the same sphere.")
+            elif np.linalg.norm(pt1 - pt2) < 1e-12:
+                raise ValueError(
+                    "Start and end represent the same point."
+                    " No arc can be formed."
+                )
+            elif np.linalg.norm(pt1 + pt2) < 1e-12:
+                raise ValueError(
+                    "Start and end are diagonally opposite, no unique arc is"
+                    " possible."
+                )
+
+        check_arguments_validity()
 
         if steps is None:
             steps = int(np.linalg.norm(pt1 - pt2) * 100)
@@ -569,8 +586,52 @@ class Bloch:
         line = pt1[:, np.newaxis] * t + pt2[:, np.newaxis] * (1 - t)
         # Normalize all the points in the line so that are distance len1 from
         # the origin.
-        arc = line * len1 / np.linalg.norm(line, axis=0)
-        self._arcs.append([arc, fmt, kwargs])
+        arc = (line * len1 / np.linalg.norm(line, axis=0)).T
+
+        # Add points in the appropriate area (rear, inner or front)
+        if len1 < 1 - 1e-12:
+            self._arcs.append([np.array(arc), "inner", fmt, kwargs])
+            return
+
+        def get_plot_area(front: bool, norm: float):
+            if front:
+                return "front"
+            if norm > 1 + 1e-12:
+                return "rear"
+            return "inner"
+
+        def append_interpolation_point(point, part, norm: float):
+            if point[0] != 0:
+                # Interpolation of the junction point to ensure continuity
+                t_edge = 1 / (1 - part[-1][0] / point[0])
+                edge_point = part[-1] * t_edge + point * (1 - t_edge)
+                edge_point = edge_point * norm / np.linalg.norm(edge_point)
+                part.append(edge_point)
+                return [edge_point, point]
+            else:
+                part.append(point)
+                return [point]
+
+        front = arc[0][0] >= 0  # is it a point in the front half-space (x>0)?
+        part = []
+        for point in arc:
+            if (point[0] >= 0) == front:
+                part.append(point)
+                continue
+
+            new_part = append_interpolation_point(point, part, len1)
+
+            # Save the arc on the previous half-space
+            self._arcs.append([np.array(part),
+                               get_plot_area(front, len1), fmt, kwargs])
+
+            # Start the arc on the new half-space
+            part = new_part
+            front = not front
+
+        # Save the last arc
+        self._arcs.append([np.array(part),
+                           get_plot_area(front, len1), fmt, kwargs])
 
     def add_line(self, start, end, fmt="k", **kwargs):
         """Adds a line segment connecting two points on the bloch sphere.
@@ -594,20 +655,12 @@ class Bloch:
             when rendering this line.
         """
         if isinstance(start, Qobj):
-            pt1 = [
-                expect(sigmax(), start),
-                expect(sigmay(), start),
-                expect(sigmaz(), start),
-            ]
+            pt1 = _state_to_cartesian_coordinates(start)
         else:
             pt1 = start
 
         if isinstance(end, Qobj):
-            pt2 = [
-                expect(sigmax(), end),
-                expect(sigmay(), end),
-                expect(sigmaz(), end),
-            ]
+            pt2 = _state_to_cartesian_coordinates(end)
         else:
             pt2 = end
 
@@ -685,15 +738,17 @@ class Bloch:
         # Matplotlib did this stretching for < 3.3.0, but not above.
         if parse_version(matplotlib.__version__) >= parse_version('3.3'):
             self.axes.set_box_aspect((1, 1, 1))
-        if not self.background:
-            self.plot_axes()
 
+        self.plot_arcs("rear")
         self.plot_back()
         self.plot_points()
         self.plot_vectors()
         self.plot_lines()
-        self.plot_arcs()
+        self.plot_arcs("inner")
+        if not self.background:
+            self.plot_axes()
         self.plot_front()
+        self.plot_arcs("front")
         self.plot_axes_labels()
         self.plot_annotations()
         # Trigger an update of the Bloch sphere if it is already shown:
@@ -703,6 +758,7 @@ class Bloch:
         # back half of sphere
         u = np.linspace(0, np.pi, 25)
         v = np.linspace(0, np.pi, 25)
+        w = np.linspace(-np.pi / 2, np.pi / 2, 25)
         x = outer(cos(u), sin(v))
         y = outer(sin(u), sin(v))
         z = outer(ones(np.size(u)), cos(v))
@@ -716,13 +772,14 @@ class Bloch:
         # equator
         self.axes.plot(1.0 * cos(u), 1.0 * sin(u), zs=0, zdir='z',
                        lw=self.frame_width, color=self.frame_color)
-        self.axes.plot(1.0 * cos(u), 1.0 * sin(u), zs=0, zdir='x',
+        self.axes.plot(1.0 * cos(w), 1.0 * sin(w), zs=0, zdir='x',
                        lw=self.frame_width, color=self.frame_color)
 
     def plot_front(self):
         # front half of sphere
         u = np.linspace(-np.pi, 0, 25)
         v = np.linspace(0, np.pi, 25)
+        w = np.linspace(-3 * np.pi / 2, -np.pi / 2, 25)
         x = outer(cos(u), sin(v))
         y = outer(sin(u), sin(v))
         z = outer(ones(np.size(u)), cos(v))
@@ -737,7 +794,7 @@ class Bloch:
         self.axes.plot(1.0 * cos(u), 1.0 * sin(u),
                        zs=0, zdir='z', lw=self.frame_width,
                        color=self.frame_color)
-        self.axes.plot(1.0 * cos(u), 1.0 * sin(u),
+        self.axes.plot(1.0 * cos(w), 1.0 * sin(w),
                        zs=0, zdir='x', lw=self.frame_width,
                        color=self.frame_color)
 
@@ -819,17 +876,17 @@ class Bloch:
                 color = self._inner_point_color[k]
             elif self.point_color is not None:
                 color = self.point_color
-            elif self.point_style[k] in ['s', 'l']:
+            elif style in ['s', 'l']:
                 color = [self.point_default_color[
                     k % len(self.point_default_color)
                 ]]
-            elif self.point_style[k] == 'm':
+            elif style == 'm':
                 length = np.ceil(num_points/len(self.point_default_color))
                 color = np.tile(self.point_default_color, length.astype(int))
                 color = color[indperm]
                 color = list(color)
 
-            if self.point_style[k] in ['s', 'm']:
+            if style in ['s', 'm']:
                 self.axes.scatter(np.real(points[1][indperm]),
                                   -np.real(points[0][indperm]),
                                   np.real(points[2][indperm]),
@@ -841,7 +898,7 @@ class Bloch:
                                   zdir='z',
                                   )
 
-            elif self.point_style[k] == 'l':
+            elif style == 'l':
                 color = color[k % len(color)]
                 self.axes.plot(np.real(points[1]),
                                -np.real(points[0]),
@@ -867,9 +924,10 @@ class Bloch:
         for line, fmt, kw in self._lines:
             self.axes.plot(line[0], line[1], line[2], fmt, **kw)
 
-    def plot_arcs(self):
-        for arc, fmt, kw in self._arcs:
-            self.axes.plot(arc[1, :], -arc[0, :], arc[2, :], fmt, **kw)
+    def plot_arcs(self, plot_area: Literal["rear", "inner", "front"]):
+        for arc, area, fmt, kw in self._arcs:
+            if plot_area == area:
+                self.axes.plot(arc[:, 1], -arc[:, 0], arc[:, 2], fmt, **kw)
 
     def show(self):
         """
