@@ -26,8 +26,8 @@ from ..settings import settings as qset
 from .options import QutipOptions
 from .data import Data
 from .cy.coefficient import (
-    Coefficient, InterCoefficient, FunctionCoefficient, StrFunctionCoefficient,
-    ConjCoefficient, NormCoefficient, ConstantCoefficient
+    Coefficient, InterCoefficient, FunctionCoefficient,
+    StrFunctionCoefficient, NormCoefficient, ConstantCoefficient,
 )
 from qutip.typing import CoefficientLike
 
@@ -40,7 +40,9 @@ class StringParsingWarning(Warning):
     pass
 
 
-def _return(base, **kwargs):
+def _return(base, args=None, **kwargs):
+    if args is not None:
+        base = base.replace_arguments(args)
     return base
 
 
@@ -61,8 +63,8 @@ def coefficient(
     base: CoefficientLike,
     *,
     tlist: ArrayLike = None,
-    args: dict = {},
-    args_ctypes: dict = {},
+    args: dict = None,
+    args_ctypes: dict = None,
     order: int = 3,
     compile_opt: dict = None,
     function_style: str = None,
@@ -177,8 +179,8 @@ def coefficient(
     """
     kwargs.update({
         "tlist": tlist,
-        'args': args,
-        'args_ctypes': args_ctypes,
+        'args': args or {},
+        'args_ctypes': args_ctypes or {},
         'order': order,
         'compile_opt': compile_opt,
         'function_style': function_style,
@@ -190,6 +192,7 @@ def coefficient(
             return coefficient_builders[type_](base, **kwargs)
 
     if callable(base):
+        args = args or {}
         op = FunctionCoefficient(base, args.copy(), style=function_style)
         if not isinstance(op(0), numbers.Number):
             raise TypeError("The coefficient function must return a number")
@@ -207,7 +210,7 @@ def norm(coeff):
 def conj(coeff):
     """ return a Coefficient with is the conjugate.
     """
-    return ConjCoefficient(coeff)
+    return coeff.conj()
 
 
 def const(value):
@@ -325,7 +328,7 @@ qset.compile = CompilationOptions()
 
 
 # Version number of the Coefficient
-COEFF_VERSION = "1.2"
+COEFF_VERSION = "1.3"
 
 try:
     root = os.path.join(qset.tmproot, f"qutip_coeffs_{COEFF_VERSION}")
@@ -477,21 +480,26 @@ def make_cy_code(code, variables, constants, raw, compile_opt):
     cdef_cte = ""
     init_cte = ""
     copy_cte = ""
+    iseq_cte = ""
     for i, (name, val, ctype) in enumerate(constants):
         cdef_cte += "        {} {}\n".format(ctype, name[5:])
         copy_cte += "        out.{} = {}\n".format(name[5:], name)
         init_cte += "        {} = cte[{}]\n".format(name, i)
+        iseq_cte += "            c_other.{} == {},\n".format(name[5:], name)
     cdef_var = ""
     init_var = ""
     init_arg = ""
     replace_var = ""
     call_var = ""
     copy_var = ""
+    iseq_var = ""
     for i, (name, val, ctype) in enumerate(variables):
         cdef_var += "        str key{}\n".format(i)
         cdef_var += "        {} {}\n".format(ctype, name[5:])
         copy_var += "        out.key{} = self.key{}\n".format(i, i)
         copy_var += "        out.{} = {}\n".format(name[5:], name)
+        iseq_var += "            c_other.key{} == self.key{},\n".format(i, i)
+        iseq_var += "            c_other.{} == {},\n".format(name[5:], name)
         if not raw:
             init_var += "        self.key{} = var[{}]\n".format(i, i)
         else:
@@ -568,6 +576,17 @@ cdef class StrCoefficient(Coefficient):
     @cython.cdivision(True)
     cdef complex _call(self, double t) except *:
 {call_var}        return {code}
+
+    def __eq__(self, other):
+        if self is other:
+            return True
+
+        if type(self) is not type(other):
+            return False
+        # cast other to StrCoefficient to access cdef attributes
+        cdef StrCoefficient c_other = other
+        return all([\n{iseq_cte}{iseq_var}        ])
+
 """
     return code
 
