@@ -77,6 +77,25 @@ class QutipOptions:
             self._properties[key](self.options[key])
 
 
+def _set_default_dtype(new_dtype):
+    import qutip
+    # Catch non existing alias and unsuported types
+    new_dtype = qutip.core.data.to.parse(new_dtype)
+    for dispatcher in qutip.core.data.to.dispatchers:
+        dispatcher.rebuild_lookup()
+
+
+def _set_default_dtype_scope(new_range):
+    import qutip
+    if new_range not in ["creation", "missing", "full"]:
+        raise ValueError(
+            "'default_dtype_scope' must be one of "
+            "'creation', 'missing', 'full'"
+        )
+    for dispatcher in qutip.core.data.to.dispatchers:
+        dispatcher.rebuild_lookup()
+
+
 class CoreOptions(QutipOptions):
     """
     Options used by the core of qutip such as the tolerance of :obj:`.Qobj`
@@ -93,8 +112,8 @@ class CoreOptions(QutipOptions):
     auto_tidyup : bool
         Whether to tidyup during sparse operations.
 
-    auto_tidyup_dims : bool [False]
-        Use auto tidyup dims on multiplication, tensor, etc.
+    auto_tidyup_dims : bool [True]
+        Whether to track the structure of scalar dimensions.
         Without auto_tidyup_dims:
 
             ``basis([2, 2]).dims == [[2, 2], [1, 1]]``
@@ -131,18 +150,35 @@ class CoreOptions(QutipOptions):
           is exactly ``f(t, args)`` then ``dict`` is used. Otherwise
           ``pythonic`` is used.
 
-    default_dtype : Nonetype, str, type {None}
-        When set, functions creating :obj:`.Qobj`, such as :func:"qeye" or
-        :func:"rand_herm", will use the specified data type. Any data-layer
-        known to ``qutip.data.to`` is accepted. When ``None``, these functions
-        will default to a sensible data type.
+    default_dtype : str, type {"core"}
+        The specified data type will be used as output for different qutip
+        functions determined by the "default_dtype_scope" options. Any
+        data-layer known to ``qutip.data.to`` is accepted. The default `core``,
+        refers to any format available in the qutip package and
+        functions will default to a sensible data type.
+
+    default_dtype_scope : str {"creation"}
+        Control where the default_dtype apply.
+
+        - "creation": Used in function creating :obj:`.Qobj`, such as
+          :func:"qeye" or :func:"rand_herm". Also used when creating `Qobj`
+          from list.
+
+        - "missing": "default_dtype" is also used in operations between data
+          layers where no exact operation match. For example ``Dense + Dense``
+          will return a ``Dense``, but ``Dense + CSR`` will use the default
+          type.
+
+        - "full": "default_dtype" is used for the output of Qobj operations and
+          forced when creating any Qobj. Be careful as it can affect the speed
+          of operation greatly.
     """
 
     _options = {
         # use auto tidyup
         "auto_tidyup": True,
-        # use auto tidyup dims on multiplication
-        "auto_tidyup_dims": False,
+        # use auto tidyup dims
+        "auto_tidyup_dims": True,
         # general absolute tolerance
         "atol": 1e-12,
         # general relative tolerance
@@ -152,7 +188,12 @@ class CoreOptions(QutipOptions):
         # signature style expected by function coefficients
         "function_coefficient_style": "auto",
         # Default Qobj dtype for Qobj create function
-        "default_dtype": None,
+        "default_dtype": "core",
+        # Where the default_dtype apply:
+        # - "creation": Used in functions creating Qobj.
+        # - "missing": Missing specialisation output use default.
+        # - "full": All data layer operation output that type.
+        "default_dtype_scope": "creation",
         # Expect, trace, etc. will return real for hermitian matrices.
         # Hermiticity checks can be slow, stop jitting, etc.
         "auto_real_casting": True,
@@ -177,7 +218,7 @@ class CoreOptions(QutipOptions):
 
     @overload
     def __getitem__(
-        self, key: Literal["function_coefficient_style"]
+        self, key: Literal["function_coefficient_style", "default_dtype_scope"]
     ) -> str: ...
 
     @overload
@@ -201,7 +242,9 @@ class CoreOptions(QutipOptions):
 
     @overload
     def __setitem__(
-        self, key: Literal["function_coefficient_style"], value: str
+        self,
+        key: Literal["function_coefficient_style", "default_dtype_scope"],
+        value: str
     ) -> None: ...
 
     @overload
@@ -215,5 +258,7 @@ class CoreOptions(QutipOptions):
 
 
 # Creating the instance of core options to use everywhere.
-# settings.core = CoreOptions()
 CoreOptions()._set_as_global_default()
+# Add properties after initial setup to not trigger the lookup rebuild
+CoreOptions._properties["default_dtype"] = _set_default_dtype
+CoreOptions._properties["default_dtype_scope"] = _set_default_dtype_scope
