@@ -2,15 +2,14 @@ __all__ = ['entropy_vn', 'entropy_linear', 'entropy_mutual', 'negativity',
            'concurrence', 'entropy_conditional', 'entangling_power',
            'entropy_relative']
 
-from numpy import conj, e, inf, imag, inner, real, sort, sqrt
-from numpy.lib.scimath import log, log2
+from .core.numpy_backend import np
 from .partial_transpose import partial_transpose
 from . import (ptrace, tensor, sigmay, ket2dm,
                expand_operator)
 from .core import data as _data
 
 
-def entropy_vn(rho, base=e, sparse=False):
+def entropy_vn(rho, base=np.e, sparse=False):
     """
     Von-Neumann entropy of density matrix
 
@@ -38,14 +37,15 @@ def entropy_vn(rho, base=e, sparse=False):
     if rho.type == 'ket' or rho.type == 'bra':
         rho = ket2dm(rho)
     vals = rho.eigenenergies(sparse=sparse)
-    nzvals = vals[vals != 0]
+    threshold = 1e-17
+    nzvals = np.where(vals < threshold, threshold, vals)
     if base == 2:
-        logvals = log2(nzvals)
-    elif base == e:
-        logvals = log(nzvals)
+        logvals = np.log2(nzvals)
+    elif base == np.e:
+        logvals = np.log(nzvals)
     else:
         raise ValueError("Base must be 2 or e.")
-    return float(real(-sum(nzvals * logvals)))
+    return np.real(-sum(nzvals * logvals))
 
 
 def entropy_linear(rho):
@@ -71,7 +71,7 @@ def entropy_linear(rho):
     """
     if rho.type == 'ket' or rho.type == 'bra':
         rho = ket2dm(rho)
-    return float(real(1.0 - (rho ** 2).tr()))
+    return np.real(1.0 - (rho ** 2).tr())
 
 
 def concurrence(rho):
@@ -113,11 +113,11 @@ def concurrence(rho):
     evals = rho_tilde.eigenenergies()
 
     # abs to avoid problems with sqrt for very small negative numbers
-    evals = abs(sort(real(evals)))
+    evals = abs(np.sort(np.real(evals)))
 
-    lsum = sqrt(evals[3]) - sqrt(evals[2]) - sqrt(evals[1]) - sqrt(evals[0])
-
-    return max(0, lsum)
+    sqrt_evals = np.sqrt(evals)
+    lsum = sqrt_evals[3] - sqrt_evals[2] - sqrt_evals[1] - sqrt_evals[0]
+    return np.maximum(0, lsum)
 
 
 def negativity(rho, subsys, method='tracenorm', logarithmic=False):
@@ -145,12 +145,12 @@ def negativity(rho, subsys, method='tracenorm', logarithmic=False):
 
 # Return the negativity value (or its logarithm if specified)
     if logarithmic:
-        return log2(2 * N + 1)
+        return np.log2(2 * N + 1)
     else:
         return N
 
 
-def entropy_mutual(rho, selA, selB, base=e, sparse=False):
+def entropy_mutual(rho, selA, selB, base=np.e, sparse=False):
     """
     Calculates the mutual information S(A:B) between selection
     components of a system density matrix.
@@ -192,7 +192,7 @@ def entropy_mutual(rho, selA, selB, base=e, sparse=False):
     return out
 
 
-def entropy_relative(rho, sigma, base=e, sparse=False, tol=1e-12):
+def entropy_relative(rho, sigma, base=np.e, sparse=False, tol=1e-12):
     """
     Calculates the relative entropy S(rho||sigma) between two density
     matrices.
@@ -253,9 +253,9 @@ def entropy_relative(rho, sigma, base=e, sparse=False, tol=1e-12):
     if rho.dims != sigma.dims:
         raise ValueError("Inputs must have the same shape and dims.")
     if base == 2:
-        log_base = log2
-    elif base == e:
-        log_base = log
+        log_base = np.log2
+    elif base == np.e:
+        log_base = np.log
     else:
         raise ValueError("Base must be 2 or e.")
     # S(rho || sigma) = sum_i(p_i log p_i) - sum_ij(p_i P_ij log q_i)
@@ -264,19 +264,19 @@ def entropy_relative(rho, sigma, base=e, sparse=False, tol=1e-12):
     # intersection with the support of rho (i.e. rvecs[rvals != 0]).
     rvals, rvecs = _data.eigs(rho.data, rho.isherm, True)
     rvecs = rvecs.to_array().T
-    if any(abs(imag(rvals)) >= tol):
+    if any(abs(np.imag(rvals)) >= tol):
         raise ValueError("Input rho has non-real eigenvalues.")
-    rvals = real(rvals)
+    rvals = np.real(rvals)
     svals, svecs = _data.eigs(sigma.data, sigma.isherm, True)
     svecs = svecs.to_array().T
-    if any(abs(imag(svals)) >= tol):
+    if any(abs(np.imag(svals)) >= tol):
         raise ValueError("Input sigma has non-real eigenvalues.")
-    svals = real(svals)
+    svals = np.real(svals)
     # Calculate inner products of eigenvectors and return +inf if kernel
     # of sigma overlaps with support of rho.
-    P = abs(inner(rvecs, conj(svecs))) ** 2
+    P = abs(np.inner(rvecs, np.conj(svecs))) ** 2
     if (rvals >= tol) @ (P >= tol) @ (svals < tol):
-        return inf
+        return np.inf
     # Avoid -inf from log(0) -- these terms will be multiplied by zero later
     # anyway
     svals[abs(svals) < tol] = 1
@@ -285,10 +285,10 @@ def entropy_relative(rho, sigma, base=e, sparse=False, tol=1e-12):
     S = nzrvals @ log_base(nzrvals) - rvals @ P @ log_base(svals)
     # the relative entropy is guaranteed to be >= 0, so we clamp the
     # calculated value to 0 to avoid small violations of the lower bound.
-    return max(0, S)
+    return np.maximum(0, S)
 
 
-def entropy_conditional(rho, selB, base=e, sparse=False):
+def entropy_conditional(rho, selB, base=np.e, sparse=False):
     """
     Calculates the conditional entropy :math:`S(A|B)=S(A,B)-S(B)`
     of a selected density matrix component.
@@ -372,9 +372,9 @@ def entangling_power(U):
         raise Exception("U must be a two-qubit gate.")
 
     from qutip.core.gates import swap
-    swap13 =  expand_operator(swap(), [2, 2, 2, 2], [1, 3])
+    swap13 =  expand_operator(swap(dtype=U.dtype), [2, 2, 2, 2], [1, 3])
     a = tensor(U, U).dag() * swap13 * tensor(U, U) * swap13
-    Uswap = swap() * U
+    Uswap = swap(dtype=U.dtype) * U
     b = tensor(Uswap, Uswap).dag() * swap13 * tensor(Uswap, Uswap) * swap13
 
     return 5.0/9 - 1.0/36 * (a.tr() + b.tr()).real
