@@ -4,8 +4,9 @@ tidyup functionality, etc.
 """
 import os
 import sys
-from ctypes import cdll
+from ctypes import cdll, CDLL
 import platform
+from glob import glob
 import numpy as np
 
 __all__ = ['settings']
@@ -34,7 +35,7 @@ def _blas_info():
     return blas
 
 
-def available_cpu_count():
+def available_cpu_count() -> int:
     """
     Get the number of cpus.
     It tries to only get the number available to qutip.
@@ -75,47 +76,57 @@ def available_cpu_count():
 
 def _find_mkl():
     """
-    Finds the MKL runtime library for the Anaconda and Intel Python
+    Finds the MKL library for the Anaconda and Intel Python
     distributions.
     """
-    mkl_lib = None
-    if _blas_info() == 'INTEL MKL':
-        plat = sys.platform
-        python_dir = os.path.dirname(sys.executable)
-        if plat in ['darwin', 'linux2', 'linux']:
-            python_dir = os.path.dirname(python_dir)
+    plat = sys.platform
 
-        if plat == 'darwin':
-            lib = '/libmkl_rt.dylib'
-        elif plat == 'win32':
-            lib = r'\mkl_rt.dll'
-        elif plat in ['linux2', 'linux']:
-            lib = '/libmkl_rt.so'
-        else:
-            raise Exception('Unknown platfrom.')
+    if plat.startswith("win"):
+        # TODO: fix the mkl handling on windows or use modules like pydiso to
+        # do it for us.
+        return ""
+    if plat == "emscripten":
+        # mkl is not supported on emscripten
+        return ""
 
-        if plat in ['darwin', 'linux2', 'linux']:
-            lib_dir = '/lib'
-        else:
-            lib_dir = r'\Library\bin'
-        # Try in default Anaconda location first
-        try:
-            mkl_lib = cdll.LoadLibrary(python_dir+lib_dir+lib)
-        except Exception:
-            pass
+    python_dir = os.path.dirname(sys.executable)
+    if plat in ['darwin', 'linux2', 'linux']:
+        python_dir = os.path.dirname(python_dir)
 
+    if plat == 'darwin':
+        ext = ".dylib"
+    elif plat == 'win32':
+        ext = ".dll"
+    elif plat in ['linux2', 'linux']:
+        ext = ".so"
+    else:
+        raise Exception('Unknown platfrom.')
+
+    # Try in default Anaconda location first
+    if plat in ['darwin', 'linux2', 'linux']:
+        lib_dir = '/lib/*'
+    else:
+        lib_dir = r'\Library\bin\*'
+
+    libraries = glob(python_dir + lib_dir)
+    mkl_libs = [lib for lib in libraries if "mkl_rt" in lib]
+
+    if not mkl_libs:
         # Look in Intel Python distro location
-        if mkl_lib is None:
-            if plat in ['darwin', 'linux2', 'linux']:
-                lib_dir = '/ext/lib'
-            else:
-                lib_dir = r'\ext\lib'
-            try:
-                mkl_lib = \
-                    cdll.LoadLibrary(python_dir + lib_dir + lib)
-            except Exception:
-                pass
-    return mkl_lib
+        if plat in ['darwin', 'linux2', 'linux']:
+            lib_dir = '/ext/lib'
+        else:
+            lib_dir = r'\ext\lib'
+        libraries = glob(python_dir + lib_dir)
+        mkl_libs = [
+            lib for lib in libraries
+            if "mkl_rt." in lib and ext in lib
+        ]
+
+    if mkl_libs:
+        # If multiple libs are found, they should all be the same.
+        return mkl_libs[-1]
+    return ""
 
 
 class Settings:
@@ -124,6 +135,7 @@ class Settings:
     """
     def __init__(self):
         self._mkl_lib = ""
+        self._mkl_lib_loc = ""
         try:
             self.tmproot = os.path.join(os.path.expanduser("~"), '.qutip')
         except OSError:
@@ -135,19 +147,60 @@ class Settings:
         self._colorblind_safe = False
 
     @property
-    def has_mkl(self):
+    def has_mkl(self) -> bool:
         """ Whether qutip found an mkl installation. """
         return self.mkl_lib is not None
 
     @property
-    def mkl_lib(self):
-        """ Location of the mkl installation. """
-        if self._mkl_lib == "":
-            self._mkl_lib = _find_mkl()
-        return _find_mkl()
+    def mkl_lib_location(self) -> str | None:
+        """ Location of the mkl library file. The file is usually called:
+
+        - `libmkl_rt.so` (Linux)
+        - `libmkl_rt.dylib` (Mac)
+        - `mkl_rt.dll` (Windows)
+
+        It search for the library in the python lib path per default.
+        If the library is in other location, update this variable as needed.
+        """
+        if self._mkl_lib_loc == "":
+            _mkl_lib_loc = _find_mkl()
+            try:
+                _mkl_lib = cdll.LoadLibrary(_mkl_lib_loc)
+            except OSError:
+                _mkl_lib = None
+            if not (
+                hasattr(_mkl_lib, "pardiso")
+                and hasattr(_mkl_lib, "mkl_cspblas_zcsrgemv")
+            ):
+                self._mkl_lib_loc = None
+                self._mkl_lib = None
+            else:
+                self._mkl_lib = _mkl_lib
+                self._mkl_lib_loc = _mkl_lib_loc
+        return self._mkl_lib_loc
+
+    @mkl_lib_location.setter
+    def mkl_lib_location(self, new: str):
+        _mkl_lib = cdll.LoadLibrary(new)
+        if not (
+            hasattr(_mkl_lib, "pardiso")
+            and hasattr(_mkl_lib, "mkl_cspblas_zcsrgemv")
+        ):
+            raise ValueError(
+                "mkl sparse functions not available in the provided library"
+            )
+        self._mkl_lib_loc = new
+        self._mkl_lib = _mkl_lib
 
     @property
-    def ipython(self):
+    def mkl_lib(self) -> CDLL | None:
+        """ Mkl library """
+        if self._mkl_lib == "":
+            self.mkl_lib_location
+        return self._mkl_lib
+
+    @property
+    def ipython(self) -> bool:
         """ Whether qutip is running in ipython. """
         try:
             __IPYTHON__
@@ -156,7 +209,7 @@ class Settings:
             return False
 
     @property
-    def eigh_unsafe(self):
+    def eigh_unsafe(self) -> bool:
         """
         Whether `eigh` call is reliable.
         Some implementation of blas have some issues on some OS.
@@ -175,7 +228,7 @@ class Settings:
         )
 
     @property
-    def tmproot(self):
+    def tmproot(self) -> str:
         """
         Location in which qutip place cython string coefficient folders.
         The default is "$HOME/.qutip".
@@ -184,13 +237,13 @@ class Settings:
         return self._tmproot
 
     @tmproot.setter
-    def tmproot(self, root):
+    def tmproot(self, root: str) -> None:
         if not os.path.exists(root):
             os.mkdir(root)
         self._tmproot = root
 
     @property
-    def coeffroot(self):
+    def coeffroot(self) -> str:
         """
         Location in which qutip save cython string coefficient files.
         Usually "{qutip.settings.tmproot}/qutip_coeffs_X.X".
@@ -199,7 +252,7 @@ class Settings:
         return self._coeffroot
 
     @coeffroot.setter
-    def coeffroot(self, root):
+    def coeffroot(self, root: str) -> None:
         if not os.path.exists(root):
             os.mkdir(root)
         if root not in sys.path:
@@ -207,18 +260,18 @@ class Settings:
         self._coeffroot = root
 
     @property
-    def coeff_write_ok(self):
+    def coeff_write_ok(self) -> bool:
         """ Whether qutip has write acces to ``qutip.settings.coeffroot``."""
         return os.access(self.coeffroot, os.W_OK)
 
     @property
-    def has_openmp(self):
+    def _has_openmp(self) -> bool:
         return False
         # We keep this as a reminder for when openmp is restored: see Pull #652
         # os.environ['KMP_DUPLICATE_LIB_OK'] = 'True'
 
     @property
-    def idxint_size(self):
+    def idxint_size(self) -> int:
         """
         Integer type used by ``CSR`` data.
         Sparse ``CSR`` matrices can contain at most ``2**idxint_size``
@@ -228,7 +281,7 @@ class Settings:
         return data.base.idxint_size
 
     @property
-    def num_cpus(self):
+    def num_cpus(self) -> int:
         """
         Number of cpu detected.
         Use the solver options to control the number of cpus used.
@@ -241,33 +294,7 @@ class Settings:
         return num_cpus
 
     @property
-    def debug(self):
-        """
-        Debug mode for development.
-        """
-        return self._debug
-
-    @debug.setter
-    def debug(self, value):
-        self._debug = value
-
-    @property
-    def log_handler(self):
-        """
-        Define whether log handler should be:
-        - default: switch based on IPython detection
-        - stream: set up non-propagating StreamHandler
-        - basic: call basicConfig
-        - null: leave logging to the user
-        """
-        return self._log_handler
-
-    @log_handler.setter
-    def log_handler(self, value):
-        self._log_handler = value
-
-    @property
-    def colorblind_safe(self):
+    def colorblind_safe(self) -> bool:
         """
         Allow for a colorblind mode that uses different colormaps
         and plotting options by default.
@@ -275,10 +302,10 @@ class Settings:
         return self._colorblind_safe
 
     @colorblind_safe.setter
-    def colorblind_safe(self, value):
+    def colorblind_safe(self, value: bool) -> None:
         self._colorblind_safe = value
 
-    def __str__(self):
+    def __str__(self) -> str:
         lines = ["Qutip settings:"]
         for attr in self.__dir__():
             if not attr.startswith('_') and attr not in ["core", "compile"]:
@@ -286,7 +313,7 @@ class Settings:
         lines.append(f"    compile: {self.compile.__repr__(full=False)}")
         return '\n'.join(lines)
 
-    def __repr__(self):
+    def __repr__(self) -> str:
         return self.__str__()
 
 
