@@ -370,6 +370,7 @@ class NonMarkovianMCSolver(MCSolver):
         "norm_steps": 5,
         "norm_t_tol": 1e-6,
         "norm_tol": 1e-4,
+        "norm_min_step": 0.0,
         "improved_sampling": False,
         "completeness_rtol": 1e-5,
         "completeness_atol": 1e-8,
@@ -653,6 +654,13 @@ class NonMarkovianMCSolver(MCSolver):
         norm_steps: int, default: 5
             Maximum number of tries to find the collapse.
 
+        norm_min_step: float, default: 0.0
+            Minimum step used when finding the collapse time, given as a
+            fraction of the search interval. Must be between 0 and 0.5.
+            A small non-zero value can help avoid the worst cases of
+            convergence at the cost of increased average steps required to find
+            the collapse.
+
         improved_sampling: Bool, default: False
             Whether to use the improved sampling algorithm
             of Abdelhafez et al. PRA (2019)
