@@ -102,7 +102,7 @@ def sesolve(
           | Maximum number of (internally defined) steps allowed in one ``tlist``
             step.
         - | max_step : float
-          | Maximum lenght of one internal step. When using pulses, it should be
+          | Maximum length of one internal step. When using pulses, it should be
             less than half the width of the thinnest pulse.
 
         Other options could be supported depending on the integration method,
