@@ -1,39 +1,50 @@
+# Required for Sphinx to follow autodoc_type_aliases
+from __future__ import annotations
+
 __all__ = ["smesolve", "SMESolver", "ssesolve", "SSESolver"]
 
+import numpy as np
+from numpy.typing import ArrayLike
+from numpy.random import SeedSequence
+from typing import Any, Callable, Literal, overload
+from functools import partial
+from time import time
+from collections.abc import Sequence
+from .multitrajresult import MultiTrajResult
 from .sode.ssystem import StochasticOpenSystem, StochasticClosedSystem
-from .result import MultiTrajResult, Result, ExpectOp
+from .sode._noise import PreSetWiener
+from .result import Result, ExpectOp
 from .multitraj import _MultiTrajRHS, MultiTrajSolver
 from .. import Qobj, QobjEvo
 from ..core.dimensions import Dimensions
-import numpy as np
-from functools import partial
+from ..core import data as _data
 from .solver_base import _solver_deprecation
 from ._feedback import _QobjFeedback, _DataFeedback, _WienerFeedback
+from ..typing import QobjEvoLike, EopsLike
 
 
 class StochasticTrajResult(Result):
     def _post_init(self, m_ops=(), dw_factor=(), heterodyne=False):
         super()._post_init()
-        self.W = []
-        self.m_ops = []
-        self.m_expect = []
-        self.dW_factor = dw_factor
+        self.noise = []
         self.heterodyne = heterodyne
-        for op in m_ops:
-            f = self._e_op_func(op)
-            self.W.append([0.0])
-            self.m_expect.append([])
-            self.m_ops.append(ExpectOp(op, f, self.m_expect[-1].append))
-            self.add_processor(self.m_ops[-1]._store)
+        if self.options["store_measurement"]:
+            self.m_ops = []
+            self.m_expect = []
+            self.dW_factor = dw_factor
+            for op in m_ops:
+                f = self._e_op_func(op)
+                self.m_expect.append([])
+                self.m_ops.append(ExpectOp(op, f, self.m_expect[-1].append))
+                self.add_processor(self.m_ops[-1]._store)
 
     def add(self, t, state, noise=None):
         super().add(t, state)
-        if noise is not None and self.options["store_measurement"]:
-            for i, dW in enumerate(noise):
-                self.W[i].append(self.W[i][-1] + dW)
+        if noise is not None:
+            self.noise.append(noise)
 
     @property
-    def wiener_process(self):
+    def wiener_process(self) -> np.typing.NDArray[float]:
         """
         Wiener processes for each stochastic collapse operators.
 
@@ -43,13 +54,17 @@ class StochasticTrajResult(Result):
             (len(sc_ops), 2, len(tlist))
         for heterodyne detection.
         """
-        W = np.array(self.W)
+        W = np.zeros(
+            (self.noise[0].shape[0], len(self.times)),
+            dtype=np.float64
+        )
+        np.cumsum(np.array(self.noise).T, axis=1, out=W[:, 1:])
         if self.heterodyne:
             W = W.reshape(-1, 2, W.shape[1])
         return W
 
     @property
-    def dW(self):
+    def dW(self) -> np.typing.NDArray[float]:
         """
         Wiener increment for each stochastic collapse operators.
 
@@ -59,13 +74,13 @@ class StochasticTrajResult(Result):
             (len(sc_ops), 2, len(tlist)-1)
         for heterodyne detection.
         """
-        dw = np.diff(self.W, axis=1)
+        noise = np.array(self.noise).T
         if self.heterodyne:
-            dw = dw.reshape(-1, 2, dw.shape[1])
-        return dw
+            return noise.reshape(-1, 2, noise.shape[1])
+        return noise
 
     @property
-    def measurement(self):
+    def measurement(self) -> np.typing.NDArray[float]:
         """
         Measurements for each stochastic collapse operators.
 
@@ -75,20 +90,41 @@ class StochasticTrajResult(Result):
             (len(sc_ops), 2, len(tlist)-1)
         for heterodyne detection.
         """
-        dts = np.diff(self.times)
-        m_expect = np.array(self.m_expect)[:, 1:]
-        noise = np.einsum(
-            "i,ij,j->ij", self.dW_factor, np.diff(self.W, axis=1), (1 / dts)
+        if not self.options["store_measurement"]:
+            return None
+        elif len(self.m_ops) == 0:
+            if self.heterodyne:
+                return np.empty(shape=(0, 2, len(self.times) - 1))
+            else:
+                return np.empty(shape=(0, len(self.times) - 1))
+        elif self.options["store_measurement"] == "start":
+            m_expect = np.array(self.m_expect)[:, :-1]
+        elif self.options["store_measurement"] == "middle":
+            m_expect = np.apply_along_axis(
+                lambda m: np.convolve(m, [0.5, 0.5], "valid"),
+                axis=1, arr=self.m_expect,
+            )
+        elif self.options["store_measurement"] in ["end", True]:
+            m_expect = np.array(self.m_expect)[:, 1:]
+        else:
+            raise ValueError(
+                "store_measurement must be in {'start', 'middle', 'end', ''}, "
+                f"not {self.options['store_measurement']}"
+            )
+        noise = np.array(self.noise).T
+        noise_scaled = np.einsum(
+            "i,ij,j->ij", self.dW_factor, noise, (1 / np.diff(self.times))
         )
         if self.heterodyne:
             m_expect = m_expect.reshape(-1, 2, m_expect.shape[1])
-            noise = noise.reshape(-1, 2, noise.shape[1])
-        return m_expect + noise
+            noise_scaled = noise_scaled.reshape(-1, 2, noise_scaled.shape[1])
+        return m_expect + noise_scaled
 
 
 class StochasticResult(MultiTrajResult):
-    def _post_init(self):
+    def _post_init(self, heterodyne=False):
         super()._post_init()
+        self.heterodyne = heterodyne
 
         store_measurement = self.options["store_measurement"]
         keep_runs = self.options["keep_runs_results"]
@@ -105,11 +141,12 @@ class StochasticResult(MultiTrajResult):
             self.add_processor(partial(self._reduce_attr, attr="measurement"))
             self._measurement = []
 
-    def _reduce_attr(self, trajectory, attr):
+    def _reduce_attr(self, trajectory, attr, *, rel=None, abs=None):
         """
         Add a result attribute to a list when the trajectories are not stored.
         """
-        getattr(self, "_" + attr).append(getattr(trajectory, attr))
+        if abs is None:
+            getattr(self, "_" + attr).append(getattr(trajectory, attr))
 
     def _trajectories_attr(self, attr):
         """
@@ -125,7 +162,7 @@ class StochasticResult(MultiTrajResult):
         return None
 
     @property
-    def measurement(self):
+    def measurement(self) -> np.typing.NDArray[float]:
         """
         Measurements for each trajectories and stochastic collapse operators.
 
@@ -138,7 +175,7 @@ class StochasticResult(MultiTrajResult):
         return self._trajectories_attr("measurement")
 
     @property
-    def dW(self):
+    def dW(self) -> np.typing.NDArray[float]:
         """
         Wiener increment for each trajectories and stochastic collapse
         operators.
@@ -152,7 +189,7 @@ class StochasticResult(MultiTrajResult):
         return self._trajectories_attr("dW")
 
     @property
-    def wiener_process(self):
+    def wiener_process(self) -> np.typing.NDArray[float]:
         """
         Wiener processes for each trajectories and stochastic collapse
         operators.
@@ -165,6 +202,34 @@ class StochasticResult(MultiTrajResult):
         """
         return self._trajectories_attr("wiener_process")
 
+    def merge(self, other: "StochasticResult", p: float = None) -> "StochasticResult":
+        if not isinstance(other, StochasticResult):
+            return NotImplemented
+        if self.stats["solver"] != other.stats["solver"]:
+            raise ValueError("Can't merge smesolve and ssesolve results")
+        if self.heterodyne != other.heterodyne:
+            raise ValueError("Can't merge heterodyne and homodyne results")
+        if p is not None:
+            raise ValueError(
+                "Stochastic solvers does not support custom weights"
+            )
+        new = super().merge(other, p)
+
+        if (
+            self.options["store_measurement"]
+            and other.options["store_measurement"]
+            and not new.trajectories
+        ):
+            new._measurement = np.concatenate(
+                (self.measurement, other.measurement), axis=0
+            )
+            new._wiener_process = np.concatenate(
+                (self.wiener_process, other.wiener_process), axis=0
+            )
+            new._dW = np.concatenate((self.dW, other.dW), axis=0)
+
+        return new
+
 
 class _StochasticRHS(_MultiTrajRHS):
     """
@@ -244,10 +309,22 @@ class _StochasticRHS(_MultiTrajRHS):
 
 
 def smesolve(
-    H, rho0, tlist, c_ops=(), sc_ops=(), heterodyne=False, *,
-    e_ops=(), args={}, ntraj=500, options=None,
-    seeds=None, target_tol=None, timeout=None, **kwargs
-):
+    H: QobjEvoLike,
+    rho0: Qobj,
+    tlist: ArrayLike,
+    c_ops: Qobj | QobjEvo | Sequence[QobjEvoLike] = (),
+    sc_ops: Qobj | QobjEvo | Sequence[QobjEvoLike] = (),
+    heterodyne: bool = False,
+    *,
+    e_ops: EopsLike | list[EopsLike] | dict[Any, EopsLike] = None,
+    args: dict[str, Any] = None,
+    ntraj: int = 500,
+    options: dict[str, Any] = None,
+    seeds: int | SeedSequence | Sequence[int | SeedSequence] = None,
+    target_tol: float | tuple[float, float] | list[tuple[float, float]] = None,
+    timeout: float = None,
+    **kwargs
+) -> StochasticResult:
     """
     Solve stochastic master equation.
 
@@ -271,11 +348,10 @@ def smesolve(
     sc_ops : list of (:obj:`.QobjEvo`, :obj:`.QobjEvo` compatible format)
         List of stochastic collapse operators.
 
-    e_ops : : :class:`.qobj`, callable, or list, optional
-        Single operator or list of operators for which to evaluate
-        expectation values or callable or list of callable.
-        Callable signature must be, `f(t: float, state: Qobj)`.
-        See :func:`.expect` for more detail of operator expectation.
+    e_ops : :obj:`.Qobj`, callable, list or dict, optional
+        Single operator, or list or dict of operators, for which to evaluate
+        expectation values. Operator can be Qobj, QobjEvo or callables with the
+        signature `f(t: float, state: Qobj) -> Any`.
 
     args : dict, optional
         Dictionary of parameters for time-dependent Hamiltonians and
@@ -319,9 +395,10 @@ def smesolve(
           | Whether or not to store the state vectors or density matrices.
             On `None` the states will be saved if no expectation operators are
             given.
-        - | store_measurement: bool
-          | Whether to store the measurement and wiener process for each
-            trajectories.
+        - | store_measurement: str, {'start', 'middle', 'end', ''}
+          | Whether and how to store the measurement for each trajectories.
+            'start', 'middle', 'end' indicate when in the interval the
+            expectation value of the ``m_ops`` is taken.
         - | keep_runs_results : bool
           | Whether to store results from all trajectories or just store the
             averages.
@@ -362,6 +439,10 @@ def smesolve(
     """
     options = _solver_deprecation(kwargs, options, "stoc")
     H = QobjEvo(H, args=args, tlist=tlist)
+    if not isinstance(sc_ops, Sequence):
+        sc_ops = [sc_ops]
+    if not isinstance(c_ops, Sequence):
+        c_ops = [c_ops]
     c_ops = [QobjEvo(c_op, args=args, tlist=tlist) for c_op in c_ops]
     sc_ops = [QobjEvo(c_op, args=args, tlist=tlist) for c_op in sc_ops]
     sol = SMESolver(
@@ -374,10 +455,21 @@ def smesolve(
 
 
 def ssesolve(
-    H, psi0, tlist, sc_ops=(), heterodyne=False, *,
-    e_ops=(), args={}, ntraj=500, options=None,
-    seeds=None, target_tol=None, timeout=None, **kwargs
-):
+    H: QobjEvoLike,
+    psi0: Qobj,
+    tlist: ArrayLike,
+    sc_ops: QobjEvoLike | Sequence[QobjEvoLike] = (),
+    heterodyne: bool = False,
+    *,
+    e_ops: EopsLike | list[EopsLike] | dict[Any, EopsLike] = None,
+    args: dict[str, Any] = None,
+    ntraj: int = 500,
+    options: dict[str, Any] = None,
+    seeds: int | SeedSequence | Sequence[int | SeedSequence] = None,
+    target_tol: float | tuple[float, float] | list[tuple[float, float]] = None,
+    timeout: float = None,
+    **kwargs
+) -> StochasticResult:
     """
     Solve stochastic Schrodinger equation.
 
@@ -397,11 +489,10 @@ def ssesolve(
     sc_ops : list of (:obj:`.QobjEvo`, :obj:`.QobjEvo` compatible format)
         List of stochastic collapse operators.
 
-    e_ops : :class:`.qobj`, callable, or list, optional
-        Single operator or list of operators for which to evaluate
-        expectation values or callable or list of callable.
-        Callable signature must be, `f(t: float, state: Qobj)`.
-        See :func:`expect` for more detail of operator expectation.
+    e_ops : :obj:`.Qobj`, callable, list or dict, optional
+        Single operator, or list or dict of operators, for which to evaluate
+        expectation values. Operator can be Qobj, QobjEvo or callables with the
+        signature `f(t: float, state: Qobj) -> Any`.
 
     args : dict, optional
         Dictionary of parameters for time-dependent Hamiltonians and
@@ -443,9 +534,10 @@ def ssesolve(
           | Whether or not to store the state vectors or density matrices.
             On `None` the states will be saved if no expectation operators are
             given.
-        - | store_measurement: bool
-            Whether to store the measurement and wiener process, or brownian
-            noise for each trajectories.
+        - | store_measurement: str, {'start', 'middle', 'end', ''}
+          | Whether and how to store the measurement for each trajectories.
+            'start', 'middle', 'end' indicate when in the interval the
+            expectation value of the ``m_ops`` is taken.
         - | keep_runs_results : bool
           | Whether to store results from all trajectories or just store the
             averages.
@@ -486,6 +578,8 @@ def ssesolve(
     """
     options = _solver_deprecation(kwargs, options, "stoc")
     H = QobjEvo(H, args=args, tlist=tlist)
+    if not isinstance(sc_ops, Sequence):
+        sc_ops = [sc_ops]
     sc_ops = [QobjEvo(c_op, args=args, tlist=tlist) for c_op in sc_ops]
     sol = SSESolver(H, sc_ops, options=options, heterodyne=heterodyne)
     return sol.run(
@@ -500,7 +594,6 @@ class StochasticSolver(MultiTrajSolver):
     """
 
     name = "StochasticSolver"
-    _resultclass = StochasticResult
     _avail_integrators = {}
     _open = None
 
@@ -516,9 +609,18 @@ class StochasticSolver(MultiTrajSolver):
         "num_cpus": None,
         "bitgenerator": None,
         "method": "platen",
-        "store_measurement": False,
+        "store_measurement": "",
     }
 
+    def _resultclass(self, e_ops, options, solver, stats):
+        return StochasticResult(
+            e_ops,
+            options,
+            solver=solver,
+            stats=stats,
+            heterodyne=self.heterodyne,
+        )
+
     def _trajectory_resultclass(self, e_ops, options):
         return StochasticTrajResult(
             e_ops,
@@ -528,7 +630,23 @@ class StochasticSolver(MultiTrajSolver):
             heterodyne=self.heterodyne,
         )
 
-    def __init__(self, H, sc_ops, heterodyne, *, c_ops=(), options=None):
+    def _initialize_stats(self):
+        stats = super()._initialize_stats()
+        if self._open:
+            stats["solver"] = "Stochastic Master Equation Evolution"
+        else:
+            stats["solver"] = "Stochastic Schrodinger Equation Evolution"
+        return stats
+
+    def __init__(
+        self,
+        H: Qobj | QobjEvo,
+        sc_ops: Sequence[Qobj | QobjEvo],
+        heterodyne: bool,
+        *,
+        c_ops: Sequence[Qobj | QobjEvo] = (),
+        options: dict[str, Any] = None,
+    ):
         self._heterodyne = heterodyne
         if self.name == "ssesolve" and c_ops:
             raise ValueError("c_ops are not supported by ssesolve.")
@@ -546,15 +664,15 @@ class StochasticSolver(MultiTrajSolver):
             self._dW_factors = np.ones(len(sc_ops))
 
     @property
-    def heterodyne(self):
+    def heterodyne(self) -> bool:
         return self._heterodyne
 
     @property
-    def m_ops(self):
+    def m_ops(self) -> list[QobjEvo | Qobj]:
         return self._m_ops
 
     @m_ops.setter
-    def m_ops(self, new_m_ops):
+    def m_ops(self, new_m_ops: list[QobjEvo | Qobj]):
         """
         Measurements operators.
 
@@ -601,11 +719,11 @@ class StochasticSolver(MultiTrajSolver):
         self._m_ops = new_m_ops
 
     @property
-    def dW_factors(self):
+    def dW_factors(self) -> np.typing.NDArray[float]:
         return self._dW_factors
 
     @dW_factors.setter
-    def dW_factors(self, new_dW_factors):
+    def dW_factors(self, new_dW_factors: np.typing.NDArray[float]):
         """
         Scaling of the noise on the measurements.
         Default are ``1`` for homodyne and ``sqrt(1/2)`` for heterodyne.
@@ -628,6 +746,154 @@ class StochasticSolver(MultiTrajSolver):
             result.add(t, self._restore_state(state, copy=False), noise)
         return seed, result
 
+    def run_from_experiment(
+        self,
+        state: Qobj,
+        tlist: ArrayLike,
+        noise: Sequence[float],
+        *,
+        args: dict[str, Any] = None,
+        e_ops: EopsLike | list[EopsLike] | dict[Any, EopsLike] = None,
+        measurement: bool = False,
+    ):
+        """
+        Run a single trajectory from a given state and noise.
+
+        Parameters
+        ----------
+        state : Qobj
+            Initial state of the system.
+
+        tlist : array_like
+            List of times for which to evaluate the state. The tlist must
+            increase uniformly.
+
+        noise : array_like
+            Noise for each time step and each stochastic collapse operators.
+            For homodyne detection, ``noise[i, t_idx]`` is the Wiener
+            increments between ``tlist[t_idx]`` and ``tlist[t_idx+1]`` for the
+            i-th sc_ops.
+            For heterodyne detection, an extra dimension is added for the pair
+            of measurement: ``noise[i, j, t_idx]``with ``j`` in ``{0,1}``.
+
+        args : dict, optional
+            Arguments to pass to the Hamiltonian and collapse operators.
+
+        e_ops : :obj:`.Qobj`, callable, list or dict, optional
+            Single operator, or list or dict of operators, for which to
+            evaluate expectation values. Operator can be Qobj, QobjEvo or
+            callables with the signature `f(t: float, state: Qobj) -> Any`.
+
+        measurement : bool, default : False
+            Whether the passed noise is the Wiener increments ``dW`` (gaussian
+            noise with standard derivation of dt**0.5), or the measurement.
+
+            Homodyne measurement is::
+
+              noise[i][t] = dW/dt + expect(sc_ops[i] + sc_ops[i].dag, state[t])
+
+            Heterodyne measurement is::
+
+              noise[i][0][t] = dW/dt * 2**0.5
+                + expect(sc_ops[i] + sc_ops[i].dag, state[t])
+
+              noise[i][1][t] = dW/dt * 2**0.5
+                -1j * expect(sc_ops[i] - sc_ops[i].dag, state[t])
+
+            Note that this function expects the expectation values to be taken
+            at the start of the time step, corresponding to the "start" setting
+            for the "store_measurements" option.
+
+            Only available for limited integration methods.
+
+        Returns
+        -------
+        result : StochasticTrajResult
+            Result of the trajectory.
+
+        Notes
+        -----
+        Only default values of `m_ops` and `dW_factors` are supported.
+        """
+        start_time = time()
+        self._argument(args)
+        stats = self._initialize_stats()
+        dt = tlist[1] - tlist[0]
+        if not np.allclose(dt, np.diff(tlist)):
+            raise ValueError("tlist must be evenly spaced.")
+        generator = PreSetWiener(
+            noise, tlist, len(self.rhs.sc_ops), self.heterodyne, measurement
+        )
+        state0 = self._prepare_state(state)
+        try:
+            old_dt = None
+            if "dt" in self._integrator.options:
+                old_dt = self._integrator.options["dt"]
+                self._integrator.options["dt"] = dt
+            mid_time = time()
+            result = self._initialize_run_one_traj(
+                None, state0, tlist, e_ops, generator=generator
+            )
+            _, result = self._integrate_one_traj(None, tlist, result)
+        except Exception as err:
+            if old_dt is not None:
+                self._integrator.options["dt"] = old_dt
+            raise
+
+        stats['preparation time'] += mid_time - start_time
+        stats['run time'] = time() - mid_time
+        result.stats.update(stats)
+        return result
+
+    @overload
+    def step(
+        self, t: float,
+        *,
+        args: dict[str, Any],
+        copy: bool,
+        wiener_increment: Literal[False],
+    ) -> Qobj: ...
+
+    @overload
+    def step(
+        self, t: float,
+        *,
+        args: dict[str, Any],
+        copy: bool,
+        wiener_increment: Literal[True],
+    ) -> tuple[Qobj, np.typing.NDArray[float]]: ...
+
+    def step(self, t, *, args=None, copy=True, wiener_increment=False):
+        """
+        Evolve the state to ``t`` and return the state as a :obj:`.Qobj`.
+
+        Parameters
+        ----------
+        t : double
+            Time to evolve to, must be higher than the last call.
+
+        args : dict, optional
+            Update the ``args`` of the system.
+            The change is effective from the beginning of the interval.
+            Changing ``args`` can slow the evolution.
+
+        copy : bool, default: True
+            Whether to return a copy of the data or the data in the ODE solver.
+
+        wiener_increment: bool, default: False
+            Whether to return ``dW`` in addition to the state.
+        """
+        if not self._integrator._is_set:
+            raise RuntimeError("The `start` method must called first.")
+        self._argument(args)
+        _, state, dW = self._integrator.integrate(t, copy=False)
+        state = self._restore_state(state, copy=copy)
+        if wiener_increment:
+            if self.heterodyne:
+                dW = dW.reshape(-1, 2)
+            return state, dW
+        return state
+
     @classmethod
     def avail_integrators(cls):
         if cls is StochasticSolver:
@@ -638,7 +904,7 @@ class StochasticSolver(MultiTrajSolver):
         }
 
     @property
-    def options(self):
+    def options(self) -> dict[str, Any]:
         """
         Options for stochastic solver:
 
@@ -651,8 +917,10 @@ class StochasticSolver(MultiTrajSolver):
             On `None` the states will be saved if no expectation operators are
             given.
 
-        store_measurement: bool, default: False
-            Whether to store the measurement for each trajectories.
+        store_measurement: str, {'start', 'middle', 'end', ''}, default: ""
+            Whether and how to store the measurement for each trajectories.
+            'start', 'middle', 'end' indicate when in the interval the
+            expectation value of the ``m_ops`` is taken.
             Storing measurements will also store the wiener process, or
             brownian noise for each trajectories.
 
@@ -697,11 +965,14 @@ class StochasticSolver(MultiTrajSolver):
         return self._options
 
     @options.setter
-    def options(self, new_options):
+    def options(self, new_options: dict[str, Any]):
         MultiTrajSolver.options.fset(self, new_options)
 
     @classmethod
-    def WienerFeedback(cls, default=None):
+    def WienerFeedback(
+        cls,
+        default: Callable[[float], np.typing.NDArray[float]] = None,
+    ):
         """
         Wiener function of the trajectory argument for time dependent systems.
 
@@ -730,7 +1001,11 @@ class StochasticSolver(MultiTrajSolver):
         return _WienerFeedback(default)
 
     @classmethod
-    def StateFeedback(cls, default=None, raw_data=False):
+    def StateFeedback(
+        cls,
+        default: Qobj | _data.Data = None,
+        raw_data: bool = False
+    ):
         """
         State of the evolution to be used in a time-dependent operator.
 
@@ -797,7 +1072,7 @@ class SMESolver(StochasticSolver):
         "num_cpus": None,
         "bitgenerator": None,
         "method": "platen",
-        "store_measurement": False,
+        "store_measurement": "",
     }
 
 
@@ -841,5 +1116,5 @@ class SSESolver(StochasticSolver):
         "num_cpus": None,
         "bitgenerator": None,
         "method": "platen",
-        "store_measurement": False,
+        "store_measurement": "",
     }
