@@ -272,3 +272,19 @@ def test_dims_comparison():
 ])
 def test_einsum(subscripts, operands, expected):
     assert einsum(subscripts, *operands) == expected
+
+
+@pytest.mark.parametrize(["list_dims", "expected"], [
+    pytest.param([[4, 4], [1, 1, 1]], [[4, 4], [1]]),
+    pytest.param([[1, 1], [1, 1, 1]], [[1], [1]]),
+    pytest.param([[1, 1, 1, 1], [5, 5, 2, 6]], [[1], [5, 5, 2, 6]]),
+    pytest.param(
+        [[[2, 3, 5], [2, 3, 5]], [[1, 1, 1], [1, 1, 1]]],
+        [[[2, 3, 5], [2, 3, 5]], [1]]
+    )
+])
+def test_scalar_dims(list_dims, expected):
+    with qutip.CoreOptions(auto_tidyup_dims=True):
+        assert Dimensions(list_dims).as_list() == expected
+    with qutip.CoreOptions(auto_tidyup_dims=False):
+        assert Dimensions(list_dims).as_list() == list_dims
