@@ -66,24 +66,26 @@ def test_QobjData():
                          ids=["copy=True", "copy=False"])
 def test_QobjCopyArgument(original_data, copy):
     """Tests that Qobj copy argument works properly when instantiating Qobj."""
-    qobj_data = qutip.Qobj(original_data, copy=copy).data
+    with qutip.CoreOptions(default_dtype_scope="creation"):
+        # default_dtype_scope="full" would break the copy logic
+        qobj_data = qutip.Qobj(original_data, copy=copy).data
 
-    if isinstance(original_data, qutip.Qobj):
-        # Qobj copies the data of another Qobj, so we take `data` if
-        # original_data was a Qobj
-        original_data = original_data.data
+        if isinstance(original_data, qutip.Qobj):
+            # Qobj copies the data of another Qobj, so we take `data` if
+            # original_data was a Qobj
+            original_data = original_data.data
 
-    if isinstance(original_data, np.ndarray):
-        # For numpy object we compare with data's data. This should be dense so
-        # we get its data as ndarray.
-        qobj_data = qobj_data.as_ndarray()
+        if isinstance(original_data, np.ndarray):
+            # For numpy object we compare with data's data. This should be dense so
+            # we get its data as ndarray.
+            qobj_data = qobj_data.as_ndarray()
 
-        # We look at the memory and see if it is shared or not to asses wether
-        # copy argument worked or not.
-        assert np.shares_memory(qobj_data, original_data) != copy
+            # We look at the memory and see if it is shared or not to asses wether
+            # copy argument worked or not.
+            assert np.shares_memory(qobj_data, original_data) != copy
 
-    else:
-        assert (original_data is qobj_data) != copy
+        else:
+            assert (original_data is qobj_data) != copy
 
 
 def test_QobjType():
@@ -596,6 +598,16 @@ def test_QobjEigenStates():
         assert c[k] == kets[k]
 
 
+def test_QobjEigenStatesOutputType():
+    op = qutip.rand_herm(5)
+
+    _, kets = op.eigenstates(output_type='kets', phase_fix=0)
+    _, oper = op.eigenstates(output_type='oper', phase_fix=0)
+
+    assert qutip.Qobj(
+           np.hstack([vec.full() for vec in kets]), dims=[5, 5]) == oper
+
+
 def test_QobjExpm():
     "qutip.Qobj expm (dense)"
     data = _random_not_singular(15)
@@ -1157,13 +1169,13 @@ def test_no_real_casting():
                  id='mixed,unchanged'),
     pytest.param([[2, 2, 2], [1, 2, 2]], [[2, 2, 2], [1, 2, 2]],
                  id='mixed,unchanged'),
-    pytest.param([[2, 2, 1], [1, 1, 1]], [[2, 2], [1, 1]], id='ket'),
-    pytest.param([[1, 2, 1], [1, 1, 1]], [[2], [1]], id='ket'),
-    pytest.param([[2, 3, 4], [1, 1, 1]], [[2, 3, 4], [1, 1, 1]],
+    pytest.param([[2, 2, 1], [1]], [[2, 2], [1]], id='ket'),
+    pytest.param([[1, 2, 1], [1]], [[2], [1]], id='ket'),
+    pytest.param([[2, 3, 4], [1]], [[2, 3, 4], [1]],
                  id='ket,unchanged'),
-    pytest.param([[1, 1, 1], [2, 2, 1]], [[1, 1], [2, 2]], id='bra'),
-    pytest.param([[1, 1, 1], [1, 2, 1]], [[1], [2]], id='bra'),
-    pytest.param([[1, 1, 1], [2, 3, 4]], [[1, 1, 1], [2, 3, 4]],
+    pytest.param([[1], [2, 2, 1]], [[1], [2, 2]], id='bra'),
+    pytest.param([[1], [1, 2, 1]], [[1], [2]], id='bra'),
+    pytest.param([[1], [2, 3, 4]], [[1], [2, 3, 4]],
                  id='bra,unchanged'),
     pytest.param([[[2, 1, 1], [2, 1, 1]], [1]], [[[2], [2]], [1]],
                  id='operket'),
@@ -1279,3 +1291,25 @@ def test_qobj_dtype(dtype):
 def test_dtype_in_info_string(dtype):
     obj = qutip.qeye(2, dtype=dtype)
     assert dtype.lower() in str(obj).lower()
+
+
+def test_constructing_op_from_states():
+    obj = qutip.basis(2, dtype="Dense")
+    assert (obj @ obj.dag()).dtype == qutip.data.to.parse("csr")
+    obj = qutip.basis(2, 0, dtype="Dense") + qutip.basis(2, 1, dtype="Dense")
+    assert (obj @ obj.dag()).dtype == qutip.data.to.parse("Dense")
+
+    
+@pytest.mark.parametrize(["state", "expected", "kwargs"], [
+        (qutip.basis([2, 2, 2], [1, 1, 0]), "(1+0j) |110>", {}),
+        (qutip.basis([2, 2, 2], [1, 1, 0]).dag(), "(1-0j) <110|", {}),
+        (qutip.basis([5, 5, 5], [4, 0, 2]), "(1+0j) |4, 0, 2>", {}),
+        (qutip.ket("1011001"), "(1+0j) |1011001>", {}),
+        (qutip.bell_state("00"), "(0.70711+0j) |11> + (0.70711+0j) |00>",
+            {"decimal_places": 5}),
+        (0*qutip.basis(2, 0), "0", {})
+])
+def test_basis_expansion(state: qutip.Qobj, expected: str, kwargs: dict):
+    result = state.basis_expansion(**kwargs)
+
+    assert result == expected
