@@ -3,7 +3,6 @@ import numpy as np
 import qutip
 from copy import copy
 from qutip.solver.mcsolve import mcsolve, MCSolver
-from qutip.solver.solver_base import Solver
 
 
 def _return_constant(t, args):
@@ -32,10 +31,11 @@ class StatesAndExpectOutputCase:
     """
     size = 10
     h = qutip.num(size)
-    state = qutip.basis(size, size-1)
+    pure_state = qutip.basis(size, size-1)
+    mixed_state = qutip.maximally_mixed_dm(size)
     times = np.linspace(0, 1, 101)
     e_ops = [qutip.num(size)]
-    ntraj = 2000
+    ntraj = 500
 
     def _assert_states(self, result, expected, tol):
         assert hasattr(result, 'states')
@@ -53,13 +53,15 @@ class StatesAndExpectOutputCase:
             np.testing.assert_allclose(test, expected_part, rtol=tol)
 
     @pytest.mark.parametrize("improved_sampling", [True, False])
-    def test_states_and_expect(self, hamiltonian, args, c_ops, expected, tol,
-                               improved_sampling):
+    def test_states_and_expect(self, hamiltonian, state, args, c_ops,
+                               expected, tol, improved_sampling):
         options = {"store_states": True, "map": "serial",
                    "improved_sampling": improved_sampling}
-        result = mcsolve(hamiltonian, self.state, self.times, args=args,
+        result = mcsolve(hamiltonian, state, self.times, args=args,
                          c_ops=c_ops, e_ops=self.e_ops, ntraj=self.ntraj,
-                         options=options, target_tol=0.05)
+                         options=options,
+                         # target_tol not supported for mixed initial state
+                         target_tol=(0.05 if state.isket else None))
         self._assert_expect(result, expected, tol)
         self._assert_states(result, expected, tol)
 
@@ -71,8 +73,6 @@ class TestNoCollapse(StatesAndExpectOutputCase):
     """
     def pytest_generate_tests(self, metafunc):
         tol = 1e-8
-        expect = (qutip.expect(self.e_ops[0], self.state)
-                  * np.ones_like(self.times))
         hamiltonian_types = [
             (self.h, "Qobj"),
             ([self.h], "list"),
@@ -80,12 +80,22 @@ class TestNoCollapse(StatesAndExpectOutputCase):
                            args={'constant': 0}), "QobjEvo"),
             (callable_qobj(self.h), "callable"),
         ]
-        cases = [pytest.param(hamiltonian, {}, [], [expect], tol, id=id)
+        cases = [pytest.param(hamiltonian, {}, [], tol, id=id)
                  for hamiltonian, id in hamiltonian_types]
         metafunc.parametrize(
-            ['hamiltonian', 'args', 'c_ops', 'expected', 'tol'],
+            ['hamiltonian', 'args', 'c_ops', 'tol'],
             cases)
 
+        initial_state_types = [
+            (self.pure_state, "pure"),
+            (self.mixed_state, "mixed"),
+        ]
+        expect = [qutip.expect(self.e_ops[0], state) * np.ones_like(self.times)
+                  for state, _ in initial_state_types]
+        cases = [pytest.param(state, [exp], id=id)
+                 for (state, id), exp in zip(initial_state_types, expect)]
+        metafunc.parametrize(['state', 'expected'], cases)
+
     # Previously the "states_only" and "expect_only" tests were mixed in to
     # every other test case.  We move them out into the simplest set so that
     # their behaviour remains tested, but isn't repeated as often to keep test
@@ -93,20 +103,20 @@ class TestNoCollapse(StatesAndExpectOutputCase):
     # test cases, this is just testing the single-output behaviour.
 
     @pytest.mark.parametrize("improved_sampling", [True, False])
-    def test_states_only(self, hamiltonian, args, c_ops, expected, tol,
-                         improved_sampling):
+    def test_states_only(self, hamiltonian, state, args, c_ops,
+                         expected, tol, improved_sampling):
         options = {"store_states": True, "map": "serial",
                    "improved_sampling": improved_sampling}
-        result = mcsolve(hamiltonian, self.state, self.times, args=args,
+        result = mcsolve(hamiltonian, state, self.times, args=args,
                          c_ops=c_ops, e_ops=[], ntraj=self.ntraj,
                          options=options)
         self._assert_states(result, expected, tol)
 
     @pytest.mark.parametrize("improved_sampling", [True, False])
-    def test_expect_only(self, hamiltonian, args, c_ops, expected, tol,
-                         improved_sampling):
+    def test_expect_only(self, hamiltonian, state, args, c_ops,
+                         expected, tol, improved_sampling):
         options = {'map': 'serial', "improved_sampling": improved_sampling}
-        result = mcsolve(hamiltonian, self.state, self.times, args=args,
+        result = mcsolve(hamiltonian, state, self.times, args=args,
                          c_ops=c_ops, e_ops=self.e_ops, ntraj=self.ntraj,
                          options=options)
         self._assert_expect(result, expected, tol)
@@ -120,8 +130,6 @@ class TestConstantCollapse(StatesAndExpectOutputCase):
     def pytest_generate_tests(self, metafunc):
         tol = 0.25
         coupling = 0.2
-        expect = (qutip.expect(self.e_ops[0], self.state)
-                  * np.exp(-coupling * self.times))
         collapse_op = qutip.destroy(self.size)
         c_op_types = [
             (np.sqrt(coupling)*collapse_op, {}, "constant"),
@@ -129,12 +137,23 @@ class TestConstantCollapse(StatesAndExpectOutputCase):
             (callable_qobj(collapse_op, _return_constant),
              {'constant': np.sqrt(coupling)}, "function"),
         ]
-        cases = [pytest.param(self.h, args, [c_op], [expect], tol, id=id)
+        cases = [pytest.param(self.h, args, [c_op], tol, id=id)
                  for c_op, args, id in c_op_types]
         metafunc.parametrize(
-            ['hamiltonian', 'args', 'c_ops', 'expected', 'tol'],
+            ['hamiltonian', 'args', 'c_ops', 'tol'],
             cases)
 
+        initial_state_types = [
+            (self.pure_state, "pure"),
+            (self.mixed_state, "mixed"),
+        ]
+        expect = [(qutip.expect(self.e_ops[0], state)
+                   * np.exp(-coupling * self.times))
+                  for state, _ in initial_state_types]
+        cases = [pytest.param(state, [exp], id=id)
+                 for (state, id), exp in zip(initial_state_types, expect)]
+        metafunc.parametrize(['state', 'expected'], cases)
+
 
 class TestTimeDependentCollapse(StatesAndExpectOutputCase):
     """
@@ -144,8 +163,6 @@ class TestTimeDependentCollapse(StatesAndExpectOutputCase):
     def pytest_generate_tests(self, metafunc):
         tol = 0.25
         coupling = 0.2
-        expect = (qutip.expect(self.e_ops[0], self.state)
-                  * np.exp(-coupling * (1 - np.exp(-self.times))))
         collapse_op = qutip.destroy(self.size)
         collapse_args = {'constant': np.sqrt(coupling), 'rate': 0.5}
         collapse_string = 'sqrt({} * exp(-t))'.format(coupling)
@@ -153,12 +170,23 @@ class TestTimeDependentCollapse(StatesAndExpectOutputCase):
             ([collapse_op, _return_decay], collapse_args, "function"),
             ([collapse_op, collapse_string], {}, "string"),
         ]
-        cases = [pytest.param(self.h, args, [c_op], [expect], tol, id=id)
+        cases = [pytest.param(self.h, args, [c_op], tol, id=id)
                  for c_op, args, id in c_op_types]
         metafunc.parametrize(
-            ['hamiltonian', 'args', 'c_ops', 'expected', 'tol'],
+            ['hamiltonian', 'args', 'c_ops', 'tol'],
             cases)
 
+        initial_state_types = [
+            (self.pure_state, "pure"),
+            (self.mixed_state, "mixed"),
+        ]
+        expect = [(qutip.expect(self.e_ops[0], state)
+                   * np.exp(-coupling * (1 - np.exp(-self.times))))
+                  for state, _ in initial_state_types]
+        cases = [pytest.param(state, [exp], id=id)
+                 for (state, id), exp in zip(initial_state_types, expect)]
+        metafunc.parametrize(['state', 'expected'], cases)
+
 
 def test_stored_collapse_operators_and_times():
     """
@@ -180,16 +208,21 @@ def test_stored_collapse_operators_and_times():
 
 @pytest.mark.parametrize("improved_sampling", [True, False])
 @pytest.mark.parametrize("keep_runs_results", [True, False])
-def test_states_outputs(keep_runs_results, improved_sampling):
+@pytest.mark.parametrize("mixed_initial_state", [True, False])
+def test_states_outputs(keep_runs_results, improved_sampling,
+                        mixed_initial_state):
     # We're just testing the output value, so it's important whether certain
     # things are complex or real, but not what the magnitudes of constants are.
     focks = 5
-    ntraj = 5
-    a = qutip.tensor(qutip.destroy(focks), qutip.qeye(2))
-    sm = qutip.tensor(qutip.qeye(focks), qutip.sigmam())
+    ntraj = 13
+    a = qutip.destroy(focks) & qutip.qeye(2)
+    sm = qutip.qeye(focks) & qutip.sigmam()
     H = 1j*a.dag()*sm + a
     H = H + H.dag()
-    state = qutip.basis([focks, 2], [0, 1])
+    if mixed_initial_state:
+        state = qutip.maximally_mixed_dm(focks) & qutip.fock_dm(2, 1)
+    else:
+        state = qutip.basis([focks, 2], [0, 1])
     times = np.linspace(0, 10, 21)
     c_ops = [a, sm]
     data = mcsolve(H, state, times, c_ops, ntraj=ntraj,
@@ -201,6 +234,10 @@ def test_states_outputs(keep_runs_results, improved_sampling):
     assert isinstance(data.average_states[0], qutip.Qobj)
     assert data.average_states[0].norm() == pytest.approx(1.)
     assert data.average_states[0].isoper
+    if state.isket:
+        assert data.average_states[0] == qutip.ket2dm(state)
+    else:
+        assert data.average_states[0] == state
 
     assert isinstance(data.average_final_state, qutip.Qobj)
     assert data.average_final_state.norm() == pytest.approx(1.)
@@ -223,9 +260,10 @@ def test_states_outputs(keep_runs_results, improved_sampling):
         assert data.runs_final_states[0].norm() == pytest.approx(1.)
         assert data.runs_final_states[0].isket
 
-    assert isinstance(data.steady_state(), qutip.Qobj)
-    assert data.steady_state().norm() == pytest.approx(1.)
-    assert data.steady_state().isoper
+    steady_state = data.steady_state()
+    assert isinstance(steady_state, qutip.Qobj)
+    assert steady_state.norm() == pytest.approx(1.)
+    assert steady_state.isoper
 
     np.testing.assert_allclose(times, data.times)
     assert data.num_trajectories == ntraj
@@ -238,20 +276,25 @@ def test_states_outputs(keep_runs_results, improved_sampling):
 
 @pytest.mark.parametrize("improved_sampling", [True, False])
 @pytest.mark.parametrize("keep_runs_results", [True, False])
-def test_expectation_outputs(keep_runs_results, improved_sampling):
+@pytest.mark.parametrize("mixed_initial_state", [True, False])
+def test_expectation_outputs(keep_runs_results, improved_sampling,
+                             mixed_initial_state):
     # We're just testing the output value, so it's important whether certain
     # things are complex or real, but not what the magnitudes of constants are.
     focks = 5
-    ntraj = 5
-    a = qutip.tensor(qutip.destroy(focks), qutip.qeye(2))
-    sm = qutip.tensor(qutip.qeye(focks), qutip.sigmam())
+    ntraj = 13
+    a = qutip.destroy(focks) & qutip.qeye(2)
+    sm = qutip.qeye(focks) & qutip.sigmam()
     H = 1j*a.dag()*sm + a
     H = H + H.dag()
-    state = qutip.basis([focks, 2], [0, 1])
+    if mixed_initial_state:
+        state = qutip.maximally_mixed_dm(focks) & qutip.fock_dm(2, 1)
+    else:
+        state = qutip.basis([focks, 2], [0, 1])
     times = np.linspace(0, 10, 5)
     c_ops = [a, sm]
     e_ops = [a.dag()*a, sm.dag()*sm, a]
-    data = mcsolve(H, state, times, c_ops, e_ops, ntraj=ntraj,
+    data = mcsolve(H, state, times, c_ops, e_ops=e_ops, ntraj=ntraj,
                    options={"keep_runs_results": keep_runs_results,
                             'map': 'serial',
                             "improved_sampling": improved_sampling})
@@ -340,7 +383,7 @@ class TestSeeds:
         kwargs = {'c_ops': self.c_ops, 'ntraj': self.ntraj,
                   "options": {"improved_sampling": improved_sampling}}
         with pytest.raises(ValueError):
-            first = mcsolve(*args, seeds=[1], **kwargs)
+            mcsolve(*args, seeds=[1], **kwargs)
 
     @pytest.mark.parametrize("improved_sampling", [True, False])
     def test_generator(self, improved_sampling):
@@ -374,18 +417,22 @@ class TestSeeds:
 
 
 @pytest.mark.parametrize("improved_sampling", [True, False])
-def test_timeout(improved_sampling):
+@pytest.mark.parametrize("mixed_initial_state", [True, False])
+def test_timeout(improved_sampling, mixed_initial_state):
     size = 10
     ntraj = 1000
     a = qutip.destroy(size)
     H = qutip.num(size)
-    state = qutip.basis(size, size-1)
+    if mixed_initial_state:
+        state = qutip.maximally_mixed_dm(size)
+    else:
+        state = qutip.basis(size, size-1)
     times = np.linspace(0, 1.0, 100)
     coupling = 0.5
     n_th = 0.05
     c_ops = np.sqrt(coupling * (n_th + 1)) * a
     e_ops = [qutip.num(size)]
-    res = mcsolve(H, state, times, c_ops, e_ops, ntraj=ntraj,
+    res = mcsolve(H, state, times, c_ops, e_ops=e_ops, ntraj=ntraj,
                   options={'map': 'serial',
                            "improved_sampling": improved_sampling},
                   timeout=1e-6)
@@ -406,31 +453,38 @@ def test_target_tol(improved_sampling):
 
     options = {'map': 'serial', "improved_sampling": improved_sampling}
 
-    res = mcsolve(H, state, times, c_ops, e_ops, ntraj=ntraj, options=options,
-                  target_tol = 0.5)
+    res = mcsolve(H, state, times, c_ops, e_ops=e_ops, ntraj=ntraj,
+                  options=options, target_tol=0.5)
     assert res.stats['end_condition'] == 'target tolerance reached'
 
-    res = mcsolve(H, state, times, c_ops, e_ops, ntraj=ntraj, options=options,
-                  target_tol = 1e-6)
+    res = mcsolve(H, state, times, c_ops, e_ops=e_ops, ntraj=ntraj,
+                  options=options, target_tol=1e-6)
     assert res.stats['end_condition'] == 'ntraj reached'
 
 @pytest.mark.parametrize("improved_sampling", [True, False])
-def test_super_H(improved_sampling):
+@pytest.mark.parametrize("mixed_initial_state", [True, False])
+def test_super_H(improved_sampling, mixed_initial_state):
     size = 10
-    ntraj = 1000
+    ntraj = 250
     a = qutip.destroy(size)
     H = qutip.num(size)
-    state = qutip.basis(size, size-1)
+    if mixed_initial_state:
+        state = qutip.maximally_mixed_dm(size)
+    else:
+        state = qutip.basis(size, size-1)
     times = np.linspace(0, 1.0, 100)
     # Arbitrary coupling and bath temperature.
     coupling = 0.5
     n_th = 0.05
     c_ops = np.sqrt(coupling * (n_th + 1)) * a
     e_ops = [qutip.num(size)]
-    mc_expected = mcsolve(H, state, times, c_ops, e_ops, ntraj=ntraj,
-                          target_tol=0.1, options={'map': 'serial'})
-    mc = mcsolve(qutip.liouvillian(H), state, times, c_ops, e_ops, ntraj=ntraj,
-                 target_tol=0.1,
+    mc_expected = mcsolve(H, state, times, c_ops, e_ops=e_ops, ntraj=ntraj,
+                          target_tol=(0.1 if state.isket else None),
+                          options={'map': 'serial',
+                                   "improved_sampling": improved_sampling})
+    mc = mcsolve(qutip.liouvillian(H), state, times, c_ops,
+                 e_ops=e_ops, ntraj=ntraj,
+                 target_tol=(0.1 if state.isket else None),
                  options={'map': 'serial',
                           "improved_sampling": improved_sampling})
     np.testing.assert_allclose(mc_expected.expect[0], mc.expect[0], atol=0.65)
@@ -509,3 +563,96 @@ def test_feedback(func, kind):
         psi0, np.linspace(0, 3, 31), e_ops=[qutip.num(10)], ntraj=10
     )
     assert np.all(result.expect[0] > 4. - tol)
+
+
+@pytest.mark.parametrize(["initial_state", "ntraj"], [
+    pytest.param(qutip.maximally_mixed_dm(2), 5, id="dm"),
+    pytest.param([(qutip.basis(2, 0), 0.3), (qutip.basis(2, 1), 0.7)],
+                 5, id="statelist"),
+    pytest.param([(qutip.basis(2, 0), 0.3), (qutip.basis(2, 1), 0.7)],
+                 [4, 2], id="ntraj-spec"),
+    pytest.param([(qutip.basis(2, 0), 0.3),
+                  ((qutip.basis(2, 0) + qutip.basis(2, 1)).unit(), 0.7)],
+                 [4, 2], id="non-orthogonals"),
+])
+@pytest.mark.parametrize("improved_sampling", [True, False])
+def test_mixed_averaging(improved_sampling, initial_state, ntraj):
+    # we will only check that the initial state of the result equals the
+    # intended initial state exactly
+    H = qutip.sigmax()
+    tlist = [0, 1]
+    L = qutip.sigmam()
+
+    solver = qutip.MCSolver(
+        H, [L], options={'improved_sampling': improved_sampling})
+    result = solver.run(initial_state, tlist, ntraj)
+
+    if isinstance(initial_state, qutip.Qobj):
+        reference = initial_state
+    else:
+        reference = sum(p * psi.proj() for psi, p in initial_state)
+
+    assert result.states[0] == reference
+    assert result.num_trajectories == np.sum(ntraj)
+
+    assert hasattr(result, 'initial_states')
+    assert isinstance(result.initial_states, list)
+    assert all(isinstance(st, qutip.Qobj) for st in result.initial_states)
+    assert hasattr(result, 'ntraj_per_initial_state')
+    assert isinstance(result.ntraj_per_initial_state, list)
+    assert len(result.ntraj_per_initial_state) == len(result.initial_states)
+    if isinstance(ntraj, list):
+        assert result.ntraj_per_initial_state == ntraj
+    else:
+        assert sum(result.ntraj_per_initial_state) == ntraj
+    assert (
+        sum(result.deterministic_weights + result.runs_weights)
+        == pytest.approx(1.)
+    )
+
+
+@pytest.mark.parametrize("improved_sampling", [True, False])
+@pytest.mark.parametrize("p", [0, 0.25, 0.5])
+def test_mixed_equals_merged(improved_sampling, p):
+    # Running mcsolve with mixed ICs should be the same as running mcsolve
+    # multiple times and merging the results afterwards
+    initial_state1 = qutip.basis(2, 1)
+    initial_state2 = (qutip.basis(2, 1) + qutip.basis(2, 0)).unit()
+    H = qutip.sigmax()
+    L = qutip.sigmam()
+    tlist = np.linspace(0, 2, 20)
+    ntraj = [3, 9]
+
+    solver = qutip.MCSolver(
+        H, [L], options={'improved_sampling': improved_sampling})
+    mixed_result = solver.run(
+        [(initial_state1, p), (initial_state2, 1 - p)], tlist, ntraj)
+
+    # Reuse seeds, then results should be identical
+    seeds = mixed_result.seeds
+    seeds1 = seeds[:ntraj[0]]
+    seeds2 = seeds[ntraj[0]:]
+
+    pure_result1 = solver.run(initial_state1, tlist, ntraj[0], seeds=seeds1)
+    pure_result2 = solver.run(initial_state2, tlist, ntraj[1], seeds=seeds2)
+    merged_result = pure_result1.merge(pure_result2, p)
+
+    assert mixed_result.num_trajectories == sum(ntraj)
+    assert merged_result.num_trajectories == sum(ntraj)
+    for state1, state2 in zip(mixed_result.states, merged_result.states):
+        assert state1 == state2
+
+    assert hasattr(mixed_result, 'initial_states')
+    assert isinstance(mixed_result.initial_states, list)
+    assert mixed_result.initial_states == [initial_state1, initial_state2]
+    assert hasattr(mixed_result, 'ntraj_per_initial_state')
+    assert isinstance(mixed_result.ntraj_per_initial_state, list)
+    assert mixed_result.ntraj_per_initial_state == ntraj
+    assert (
+        sum(mixed_result.runs_weights + mixed_result.deterministic_weights) 
+        == pytest.approx(1.)
+    )
+    assert (
+        sum(merged_result.runs_weights + merged_result.deterministic_weights)
+        == pytest.approx(1.)
+    )
