@@ -5,7 +5,7 @@ visualizations of quantum states and processes.
 
 __all__ = ['plot_wigner_sphere', 'hinton', 'sphereplot',
            'matrix_histogram', 'plot_energy_levels', 'plot_fock_distribution',
-           'plot_wigner', 'plot_expectation_values',
+           'plot_wigner', 'plot_qfunc', 'plot_expectation_values',
            'plot_spin_distribution', 'complex_array_to_rgb',
            'plot_qubism', 'plot_schmidt']
 
@@ -19,7 +19,7 @@ from . import (
     Qobj, isket, ket2dm, tensor, vector_to_operator, settings
 )
 from .core.superop_reps import _to_superpauli, isqubitdims
-from .wigner import wigner
+from .wigner import wigner, qfunc
 from .matplotlib_utilities import complex_phase_cmap
 
 try:
@@ -978,6 +978,8 @@ def matrix_histogram(
                     ).sum(0).flatten()
         )
 
+        frame_artists = []
+
         for i, uxpos in enumerate(xpos):
             artist = ax.bar3d(
                 uxpos,
@@ -991,9 +993,10 @@ def matrix_histogram(
                 linewidths=options["bars_lw"],
                 shade=options["shade"],
             )
-            # Setting the z-order for rendering
             artist._sort_zpos = z_order[i]
-            artist_list.append([artist])
+            frame_artists.append(artist)
+
+        artist_list.append(frame_artists)
 
     if len(Ms) == 1:
         output = ax
@@ -1344,6 +1347,118 @@ def plot_wigner(rho, xvec=None, yvec=None, method='clenshaw', projection='2d',
     return fig, output
 
 
+def plot_qfunc(rho, xvec=None, yvec=None, projection='2d',
+               g=sqrt(2), *, cmap=None, colorbar=False,
+               fig=None, ax=None):
+    """
+    Plot the the Husimi-Q function for a density matrix (or ket).
+
+    Parameters
+    ----------
+    rho : :obj:`.Qobj`
+        The density matrix (or ket) of the state to visualize.
+
+    xvec : array_like, optional
+        x-coordinates at which to calculate the Husimi-Q function.
+
+    yvec : array_like, optional
+        y-coordinates at which to calculate the Husimi-Q function.
+
+    projection: str {'2d', '3d'}, default: '2d'
+        Specify whether the Husimi-Q function is to be plotted as a
+        contour graph ('2d') or surface plot ('3d').
+
+    g : float
+        Scaling factor for `a = 0.5 * g * (x + iy)`, default `g = sqrt(2)`.
+
+    cmap : a matplotlib cmap instance, optional
+        The colormap.
+
+    colorbar : bool, default: False
+        Whether (True) or not (False) a colorbar should be attached to the
+        Husimi-Q function graph.
+
+    fig : a matplotlib Figure instance, optional
+        The Figure canvas in which the plot will be drawn.
+
+    ax : a matplotlib axes instance, optional
+        The axes context in which the plot will be drawn.
+
+    Returns
+    -------
+    fig, output : tuple
+        A tuple of the matplotlib figure and the axes instance or animation
+        instance used to produce the figure.
+    """
+
+    if projection not in ('2d', '3d'):
+        raise ValueError('Unexpected value of projection keyword argument')
+
+    fig, ax = _is_fig_and_ax(fig, ax, projection)
+
+    if not isinstance(rho, list):
+        rhos = [rho]
+    else:
+        rhos = rho
+
+    _equal_shape(rhos)
+
+    wlim = 0
+    Ws = list()
+    xvec = np.linspace(-7.5, 7.5, 200) if xvec is None else xvec
+    yvec = np.linspace(-7.5, 7.5, 200) if yvec is None else yvec
+    for rho in rhos:
+        if isket(rho):
+            rho = ket2dm(rho)
+
+        W0 = qfunc(
+            rho, xvec, yvec, g=g
+        )
+
+        W, yvec = W0 if isinstance(W0, tuple) else (W0, yvec)
+        Ws.append(W)
+
+        wlim = max(abs(W).max(), wlim)
+
+    norm = mpl.colors.Normalize(-wlim, wlim)
+
+    if cmap is None:
+        cmap = _diverging_cmap()
+
+    artist_list = list()
+    for W in Ws:
+        if projection == '2d':
+            if parse_version(mpl.__version__) >= parse_version('3.8'):
+                cf = [ax.contourf(xvec, yvec, W, 100, norm=norm, cmap=cmap)]
+            else:
+                cf = ax.contourf(xvec, yvec, W, 100, norm=norm,
+                                 cmap=cmap).collections
+        else:
+            X, Y = np.meshgrid(xvec, yvec)
+            cf = [ax.plot_surface(X, Y, W, rstride=5, cstride=5, linewidth=0.5,
+                                  norm=norm, cmap=cmap)]
+        artist_list.append(cf)
+
+    if len(rhos) == 1:
+        output = ax
+    else:
+        output = animation.ArtistAnimation(fig, artist_list, interval=50,
+                                           blit=True, repeat_delay=1000)
+
+    ax.set_xlabel(r'$\rm{Re}(\alpha)$', fontsize=12)
+    ax.set_ylabel(r'$\rm{Im}(\alpha)$', fontsize=12)
+
+    if colorbar:
+        if projection == '2d':
+            shrink = 1
+        else:
+            shrink = .75
+        cax, kw = mpl.colorbar.make_axes(ax, shrink=shrink, pad=.1)
+        cbar = mpl.colorbar.ColorbarBase(cax, cmap=cmap, norm=norm)
+
+    return fig, output
+
+
 def plot_expectation_values(results, ylabels=None, *,
                             fig=None, axes=None):
     """
